==> topcl-1.1.tar.gz: topcl.c <==
/*
 * "$Id: topcl.c,v 1.4 93/12/05 10:43:57 mike Exp Locker: mike $"
 *
 * Mainline code for TOPCL, an SGI image to PCL converter.
 *
 * Copyright (c) 1993 Easy Software Products
 *                    P.O. Box 256
 *                    Park Hall, MD  20667
 *                    Attn: Michael Sweet (dodgecolt@delphi.com)
 *
 * This file is part of TOPCL.
 *
 * TOPCL is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the TOPCL General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * TOPCL, but only under the conditions described in the TOPCL
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with TOPCL so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies.
 *
 * Contents:
 *
 *  compare_string() - Compare two strings, allowing for abbreviations.
 *  lookup_string()  - Find a string in a lookup table...
 *  usage()          - Display TOPCL usage and exit.
 *  parse_options()  - Get options from command-line...
 *  set_defaults()   - Set defaults from current options.
 *  main()           - Get options and output as many pages as necessary...
 *
 * Revision History:
 *
 *  $Log:	topcl.c,v $
 * Revision 1.4  93/12/05  10:43:57  mike
 * Updated documentation for v1.1 release.
 * 
 * Revision 1.3  93/12/05  02:24:25  mike
 * Updated command-line option/name recognition to accept partial names
 *   (i.e. abbreviations) when possible.
 * 
 * Revision 1.2  93/06/10  22:34:22  mike
 * Cleaned up for version 1.0 release.
 * 
 * Revision 1.1  93/05/24  19:42:32  mike
 * Initial revision
 * 
 *
 */

/*
 * Include necessary headers.
 */

#define _TOPCL_C_
#include "topcl.h"


/*
 * Lookup tables...
 */

typedef struct
 {
  int  value; /* Integer value of 'name' */
  char *name; /* Name to match */
  int  chars; /* Number of necessary chars */
 } LOOKUP;


LOOKUP models[] =
 {
  { PCL_LJ,      "lj",      0 },
  { PCL_LJPLUS,  "lj+",     0 },
  { PCL_LJ500,   "lj500",   0 },
  { PCL_LJ2,     "lj2",     0 },
  { PCL_LJ2P,    "lj2p",    0 },
  { PCL_LJ3,     "lj3",     0 },
  { PCL_LJ4,     "lj4",     0 },
  { PCL_DJ500,   "dj500",   0 },
  { PCL_DJ500C,  "dj500c",  0 },
  { PCL_DJ550,   "dj550",   0 },
  { PCL_DJ550C,  "dj550c",  0 },
  { PCL_DJ1200C, "dj1200c", 3 },
  { -1,          NULL,      0 }
 };

LOOKUP dpis[] =
 {
  { 75,  "75",  1 },
  { 100, "100", 2 },
  { 150, "150", 2 },
  { 300, "300", 1 },
  { 600, "600", 1 },
  { -1,  NULL,  0 }
 };

LOOKUP sizes[] =
 {
  { PCL_LETTER_SIZE,    "letter",    3 },
  { PCL_LEGAL_SIZE,     "legal",     3 },
  { PCL_EXECUTIVE_SIZE, "executive", 1 },
  { PCL_A4_SIZE,        "a4",        1 },
  { -1,                 NULL,        0 }
 };

LOOKUP sources[] =
 {
  { PCL_UPPER_TRAY,  "upper",  1 },
  { PCL_LOWER_TRAY,  "lower",  1 },
  { PCL_MANUAL_FEED, "manual", 1 },
  { -1,              NULL,     0 }
 };

LOOKUP medias[] =
 {
  { PCL_PAPER,        "paper",        1 },
  { PCL_TRANSPARENCY, "transparency", 1 },
  { -1,               NULL,           0 }
 };

LOOKUP flips[] =
 {
  { PCL_FLIP_NONE, "none", 1 },
  { PCL_FLIP_X,    "x",    0 },
  { PCL_FLIP_Y,    "y",    1 },
  { PCL_FLIP_XY,   "xy",   0 },
  { -1,            NULL,   0 }
 };


/*
 * 'compare_string()' - Compare two strings, allowing for abbreviations.
 *
 * Returns 0 if the strings are equivalent, non-zero otherwise.
 */

static int
compare_string(char *s,    /* I - String to compare */
               char *name, /* I - Name to compare */
               int  chars) /* I - Minimum number of chars in 'name' that must *
                            *     match (0 if must match completely).         */
{
 int len; /* Length of string to look for */


 len = strlen(s);

 if (len < strlen(name))
  {
   if (len >= chars &&
       strncasecmp(name, s, len) == 0)
    return (0);
   else
    return (-1);
  }
 else
  return (strcasecmp(name, s));
}


/*
 * 'lookup_string()' - Find a string in a lookup table...
 */

static int
lookup_string(LOOKUP *table,   /* I - Table to look in */
              char   *string)  /* I - String to look for */
{
 if (table == NULL || string == NULL)
  return (-1);

 while (table->value >= 0)
  {
   if (compare_string(string, table->name, table->chars) == 0)
    break;
   else
    table ++;
  };

 return (table->value);
}


/*
 * 'usage()' - Display TOPCL usage and exit.
 */

static void
usage(void)
{
 fputs("\aTOPCL Version 1.1\n", stderr);
 fputs("Copyright (c) 1993 by Easy Software Products\n\n", stderr);
 fputs("Usage: topcl infile.rgb [options] >outfile.pcl\n\n", stderr);
 fputs("Options:\n", stderr);
 fputs("      -model = Select printer model:\n", stderr);
 fputs("          lj, lj+, lj500, lj2, lj2p, lj3, lj4,\n", stderr);
 fputs("          dj500, dj500c, dj550, dj550c, dj1200c.\n", stderr);
 fputs("      -bw = Select black-and-white output.\n", stderr);
 fputs("      -color = Select color output.\n", stderr);
 fputs("      -compmode = Select compression mode:\n", stderr);
 fputs("          0 = no compression,\n", stderr);
 fputs("          2 = TIFF 4.0 RLE compression,\n", stderr);
 fputs("          3 = delta-row compression.\n", stderr);
 fputs("      -dpi = Select output resolution (dots-per-inch):\n", stderr);
 fputs("          75, 100, 150, 300, 600 (lj4 only).\n", stderr);
 fputs("      -copies = Set number of copies made on output (1-99).\n", stderr);
 fputs("      -size = Set media size:\n", stderr);
 fputs("          letter, legal, executive, a4.\n", stderr);
 fputs("      -media = Set output media type:\n", stderr);
 fputs("          paper, transparency.\n", stderr);
 fputs("      -source = Set output media source:\n", stderr);
 fputs("          upper, lower, manual.\n", stderr);
 fputs("      -portrait = Set output to portrait orientation.\n", stderr);
 fputs("      -landscape = Set output to landscape orientation.\n", stderr);
 fputs("      -pages = Set output size in pages:\n", stderr);
 fputs("          N = output NxN pages, NxM = output NxM pages.\n", stderr);
 fputs("      -flip = Flip the output along the specified axis:\n", stderr);
 fputs("          none, x, y, xy.\n", stderr);
 fputs("      -gamma = Apply the specified gamma correction to the image.\n", stderr);
 fputs("\n", stderr);

 exit(1);
}


/*
 * 'parse_options()' - Get options from command-line...
 */

static void
parse_options(int  argc,    /* I - Number of options... */
              char *argv[]) /* I - Options... */
{
 int i;                     /* Looping var */


#ifdef DEBUG
 fprintf(stderr, "Entering parse_options(), %d args to parse...\n", argc);
#endif /* DEBUG */

 for (i = 0; i < argc; i ++)
  {
#ifdef DEBUG
   fprintf(stderr, "Parsing \'%s\'...\n", argv[i]);
#endif /* DEBUG */

   if (compare_string(argv[i], "-model", 3) == 0)
    {
     i ++;
     if ((PrinterModel = lookup_string(models, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-bw", 0) == 0)
    OutputType = PCL_BW;
   else if (compare_string(argv[i], "-color", 4) == 0)
    OutputType = PCL_COLOR;
   else if (compare_string(argv[i], "-compmode", 4) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputMode = atoi(argv[i])) < 0 ||
         OutputMode > 3)
      usage();
    }
   else if (compare_string(argv[i], "-dpi", 2) == 0)
    {
     i ++;
     if ((OutputDpi = lookup_string(dpis, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-copies", 4) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputCopies = atoi(argv[i])) < 0 ||
         OutputCopies > 99)
      usage();
    }
   else if (compare_string(argv[i], "-size", 3) == 0)
    {
     i ++;
     if ((PaperSize = lookup_string(sizes, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-source", 3) == 0)
    {
     i ++;
     if ((PaperSource = lookup_string(sources, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-media", 3) == 0)
    {
     i ++;
     if ((OutputMedia = lookup_string(medias, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-portrait", 3) == 0)
    OutputOrient = PCL_PORTRAIT;
   else if (compare_string(argv[i], "-landscape", 2) == 0)
    OutputOrient = PCL_LANDSCAPE;
   else if (compare_string(argv[i], "-pages", 3) == 0)
    {
     i ++;
     if ((OutputPages = argv[i]) == NULL)
      usage();
    }
   else if (compare_string(argv[i], "-flip", 2) == 0)
    {
     i ++;
     if ((OutputFlip = lookup_string(flips, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-gamma", 2) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputGamma = atof(argv[i])) < 0.0 ||
         OutputGamma > 20.0)
      usage();
    }
   else
    usage();
  };
}


/*
 * 'set_defaults()' - Set defaults from current options.
 */

static PRINTER *
set_defaults(RGB_IMAGE *i) /* I - Image to be printed */
{
 int     x;                /* Looping var... */
 PRINTER *printer;         /* Currently selected printer... */


 printer = NULL;
 for (x = 0; x < (sizeof(PrinterModels) / sizeof(PRINTER)); x ++)
  if (PrinterModels[x].model == PrinterModel)
   {
    printer = PrinterModels + x;
    break;
   };

 if (printer == NULL)
  {
   fputs("\atopcl: Internal program error during search for printer model!\n", stderr);
   exit(1);
  };

 if (OutputType == PCL_AUTO)
  OutputType = printer->type;

 if (OutputMode == PCL_AUTO)
  OutputMode = printer->mode;

 if (OutputDpi == PCL_AUTO)
  OutputDpi = printer->dpi;

 if (OutputOrient == PCL_AUTO)
  {
  /*
   * If no orientation has be selected, orient the image to have the long
   * side lie on the vertical edge (to fill the page as well as possible).
   */

   if (i->width > i->height)
    OutputOrient = PCL_LANDSCAPE;
   else
    OutputOrient = PCL_PORTRAIT;
  };

 switch (PaperSize)
  {
   case PCL_LETTER_SIZE :
       PaperWidth  = printer->letter_width;
       PaperHeight = printer->letter_height;
       break;
   case PCL_LEGAL_SIZE :
       PaperWidth  = printer->legal_width;
       PaperHeight = printer->legal_height;
       break;
   case PCL_EXECUTIVE_SIZE :
       PaperWidth  = printer->exec_width;
       PaperHeight = printer->exec_height;
       break;
   case PCL_A4_SIZE :
       PaperWidth  = printer->a4_width;
       PaperHeight = printer->a4_height;
       break;
  };

 return (printer);
}


/*
 * 'main()' - Get options and output as many pages as necessary...
 */

void
main(int  argc,        /* I - Number of command-line arguments */
     char *argv[])     /* I - Command-line arguments */
{
 RGB_IMAGE *i;         /* Image to convert */
 int       hpage,      /* Horizontal page count */
           vpage;      /* Vertical page count */
 int       num_hpages, /* Number of pages vertically */
           num_vpages; /* Number of pages horizontally */
 int       copy,       /* Current copy */
           num_copies; /* Number of copies to make */
 PRINTER   *printer;   /* Current printer */


/*
 * First check for command-line args and parse-'em...
 */

 if (argc < 2)
  usage();

 i = LoadImage(argv[1]);
#ifdef DEBUG
 fputs("Image loaded...\n", stderr);
#endif /* DEBUG */

 parse_options(argc - 2, argv + 2);

/*
 * Set any remaining defaults and prepare the image for output...
 */

 printer = set_defaults(i);
#ifdef DEBUG
 fputs("Defaults set...\n", stderr);
#endif /* DEBUG */

 if (OutputType == PCL_BW)
  GreyImage(i);

 AdjustImage(i, OutputGamma);
#ifdef DEBUG
 fputs("Image prepped...\n", stderr);
#endif /* DEBUG */


/*
 * Figure out how many pages per image to output...
 */

 if (sscanf(OutputPages, "%dx%d", &num_hpages, &num_vpages) < 2)
  { /* Only one axis specified; duplicate... */
   num_vpages = num_hpages;
  };

/*
 * Initialize the printer as necessary...
 */

 printf("\033E");                     /* Reset the printer */

 if (printer->copies)
  {
   printf("\033&l%dX", OutputCopies); /* Have the printer do the copies */
   num_copies = 1;                    /* Dither the image once */
  }
 else
  {
   num_copies = OutputCopies;         /* Dither the image 'num_copies' times */
  };

 printf("\033&l%dA", PaperSize);      /* Set paper size... */

 if (PaperSource != PCL_DEFAULT_TRAY)
  printf("\033&l%dH", PaperSource);   /* Get media from specified source... */

 printf("\033*t%dR", OutputDpi);      /* Set graphics resolution */

/*
 * Now output each page & copy as necessary...
 */

 for (copy = 0; copy < num_copies; copy ++)
  for (hpage = 0; hpage < num_hpages; hpage ++)
   for (vpage = 0; vpage < num_vpages; vpage ++)
    OutputImage(i, hpage * i->width / num_hpages,
                vpage * i->height / num_vpages,
                (hpage + 1) * (i->width - 1) / num_hpages,
                (vpage + 1) * (i->height - 1) / num_vpages);

 printf("\033E");                     /* Reset the printer */

 FreeImage(i);                        /* Free up any memory used before exit */
}


/*
 * End of "$Id: topcl.c,v 1.4 93/12/05 10:43:57 mike Exp Locker: mike $".
 */

==> topcl-1.1.1.tar.gz: topcl.c <==
/*
 * "$Id: topcl.c,v 1.5 93/12/22 16:50:39 mike Exp Locker: mike $"
 *
 * Mainline code for TOPCL, an SGI image to PCL converter.
 *
 * Copyright (c) 1993 Easy Software Products
 *                    P.O. Box 256
 *                    Park Hall, MD  20667
 *                    Attn: Michael Sweet (dodgecolt@delphi.com)
 *
 * This file is part of TOPCL.
 *
 * TOPCL is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
 * to anyone for the consequences of using it or for whether it serves any
 * particular purpose or works at all, unless he says so in writing.  Refer
 * to the TOPCL General Public License for full details.
 *
 * Everyone is granted permission to copy, modify and redistribute
 * TOPCL, but only under the conditions described in the TOPCL
 * General Public License.  A copy of this license is supposed to have been
 * given to you along with TOPCL so you can know your rights and
 * responsibilities.  It should be in a file named COPYING.  Among other
 * things, the copyright notice and this notice must be preserved on all
 * copies.
 *
 * Contents:
 *
 *  compare_string() - Compare two strings, allowing for abbreviations.
 *  lookup_string()  - Find a string in a lookup table...
 *  usage()          - Display TOPCL usage and exit.
 *  parse_options()  - Get options from command-line...
 *  set_defaults()   - Set defaults from current options.
 *  main()           - Get options and output as many pages as necessary...
 *
 * Revision History:
 *
 *  $Log:	topcl.c,v $
 * Revision 1.5  93/12/22  16:50:39  mike
 * Added -model option for the LJ4L.
 * 
 * Revision 1.4  93/12/05  10:43:57  mike
 * Updated documentation for v1.1 release.
 * 
 * Revision 1.3  93/12/05  02:24:25  mike
 * Updated command-line option/name recognition to accept partial names
 *   (i.e. abbreviations) when possible.
 * 
 * Revision 1.2  93/06/10  22:34:22  mike
 * Cleaned up for version 1.0 release.
 * 
 * Revision 1.1  93/05/24  19:42:32  mike
 * Initial revision
 * 
 *
 */

/*
 * Include necessary headers.
 */

#define _TOPCL_C_
#include "topcl.h"


/*
 * Lookup tables...
 */

typedef struct
 {
  int  value; /* Integer value of 'name' */
  char *name; /* Name to match */
  int  chars; /* Number of necessary chars */
 } LOOKUP;


LOOKUP models[] =
 {
  { PCL_LJ,      "lj",      0 },
  { PCL_LJPLUS,  "lj+",     0 },
  { PCL_LJ500,   "lj500",   0 },
  { PCL_LJ2,     "lj2",     0 },
  { PCL_LJ2P,    "lj2p",    0 },
  { PCL_LJ3,     "lj3",     0 },
  { PCL_LJ4,     "lj4",     0 },
  { PCL_LJ3,     "lj4l",    0 },
  { PCL_DJ500,   "dj500",   0 },
  { PCL_DJ500C,  "dj500c",  0 },
  { PCL_DJ550,   "dj550",   0 },
  { PCL_DJ550C,  "dj550c",  0 },
  { PCL_DJ1200C, "dj1200c", 3 },
  { -1,          NULL,      0 }
 };

LOOKUP dpis[] =
 {
  { 75,  "75",  1 },
  { 100, "100", 2 },
  { 150, "150", 2 },
  { 300, "300", 1 },
  { 600, "600", 1 },
  { -1,  NULL,  0 }
 };

LOOKUP sizes[] =
 {
  { PCL_LETTER_SIZE,    "letter",    3 },
  { PCL_LEGAL_SIZE,     "legal",     3 },
  { PCL_EXECUTIVE_SIZE, "executive", 1 },
  { PCL_A4_SIZE,        "a4",        1 },
  { -1,                 NULL,        0 }
 };

LOOKUP sources[] =
 {
  { PCL_UPPER_TRAY,  "upper",  1 },
  { PCL_LOWER_TRAY,  "lower",  1 },
  { PCL_MANUAL_FEED, "manual", 1 },
  { -1,              NULL,     0 }
 };

LOOKUP medias[] =
 {
  { PCL_PAPER,        "paper",        1 },
  { PCL_TRANSPARENCY, "transparency", 1 },
  { -1,               NULL,           0 }
 };

LOOKUP flips[] =
 {
  { PCL_FLIP_NONE, "none", 1 },
  { PCL_FLIP_X,    "x",    0 },
  { PCL_FLIP_Y,    "y",    1 },
  { PCL_FLIP_XY,   "xy",   0 },
  { -1,            NULL,   0 }
 };


/*
 * 'compare_string()' - Compare two strings, allowing for abbreviations.
 *
 * Returns 0 if the strings are equivalent, non-zero otherwise.
 */

static int
compare_string(char *s,    /* I - String to compare */
               char *name, /* I - Name to compare */
               int  chars) /* I - Minimum number of chars in 'name' that must *
                            *     match (0 if must match completely).         */
{
 int len; /* Length of string to look for */


 len = strlen(s);

 if (len < strlen(name))
  {
   if (len >= chars &&
       strncasecmp(name, s, len) == 0)
    return (0);
   else
    return (-1);
  }
 else
  return (strcasecmp(name, s));
}


/*
 * 'lookup_string()' - Find a string in a lookup table...
 */

static int
lookup_string(LOOKUP *table,   /* I - Table to look in */
              char   *string)  /* I - String to look for */
{
 if (table == NULL || string == NULL)
  return (-1);

 while (table->value >= 0)
  {
   if (compare_string(string, table->name, table->chars) == 0)
    break;
   else
    table ++;
  };

 return (table->value);
}


/*
 * 'usage()' - Display TOPCL usage and exit.
 */

static void
usage(void)
{
 fputs("\aTOPCL Version 1.1.1\n", stderr);
 fputs("Copyright (c) 1993 by Easy Software Products\n\n", stderr);
 fputs("Usage: topcl infile.rgb [options] >outfile.pcl\n\n", stderr);
 fputs("Options:\n", stderr);
 fputs("      -model = Select printer model:\n", stderr);
 fputs("          lj, lj+, lj500, lj2, lj2p, lj3, lj4, lj4l,\n", stderr);
 fputs("          dj500, dj500c, dj550, dj550c, dj1200c.\n", stderr);
 fputs("      -bw = Select black-and-white output.\n", stderr);
 fputs("      -color = Select color output.\n", stderr);
 fputs("      -compmode = Select compression mode:\n", stderr);
 fputs("          0 = no compression,\n", stderr);
 fputs("          2 = TIFF 4.0 RLE compression,\n", stderr);
 fputs("          3 = delta-row compression.\n", stderr);
 fputs("      -dpi = Select output resolution (dots-per-inch):\n", stderr);
 fputs("          75, 100, 150, 300, 600 (lj4 only).\n", stderr);
 fputs("      -copies = Set number of copies made on output (1-99).\n", stderr);
 fputs("      -size = Set media size:\n", stderr);
 fputs("          letter, legal, executive, a4.\n", stderr);
 fputs("      -media = Set output media type:\n", stderr);
 fputs("          paper, transparency.\n", stderr);
 fputs("      -source = Set output media source:\n", stderr);
 fputs("          upper, lower, manual.\n", stderr);
 fputs("      -portrait = Set output to portrait orientation.\n", stderr);
 fputs("      -landscape = Set output to landscape orientation.\n", stderr);
 fputs("      -pages = Set output size in pages:\n", stderr);
 fputs("          N = output NxN pages, NxM = output NxM pages.\n", stderr);
 fputs("      -flip = Flip the output along the specified axis:\n", stderr);
 fputs("          none, x, y, xy.\n", stderr);
 fputs("      -gamma = Apply the specified gamma correction to the image.\n", stderr);
 fputs("\n", stderr);

 exit(1);
}


/*
 * 'parse_options()' - Get options from command-line...
 */

static void
parse_options(int  argc,    /* I - Number of options... */
              char *argv[]) /* I - Options... */
{
 int i;                     /* Looping var */


#ifdef DEBUG
 fprintf(stderr, "Entering parse_options(), %d args to parse...\n", argc);
#endif /* DEBUG */

 for (i = 0; i < argc; i ++)
  {
#ifdef DEBUG
   fprintf(stderr, "Parsing \'%s\'...\n", argv[i]);
#endif /* DEBUG */

   if (compare_string(argv[i], "-model", 3) == 0)
    {
     i ++;
     if ((PrinterModel = lookup_string(models, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-bw", 0) == 0)
    OutputType = PCL_BW;
   else if (compare_string(argv[i], "-color", 4) == 0)
    OutputType = PCL_COLOR;
   else if (compare_string(argv[i], "-compmode", 4) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputMode = atoi(argv[i])) < 0 ||
         OutputMode > 3)
      usage();
    }
   else if (compare_string(argv[i], "-dpi", 2) == 0)
    {
     i ++;
     if ((OutputDpi = lookup_string(dpis, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-copies", 4) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputCopies = atoi(argv[i])) < 0 ||
         OutputCopies > 99)
      usage();
    }
   else if (compare_string(argv[i], "-size", 3) == 0)
    {
     i ++;
     if ((PaperSize = lookup_string(sizes, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-source", 3) == 0)
    {
     i ++;
     if ((PaperSource = lookup_string(sources, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-media", 3) == 0)
    {
     i ++;
     if ((OutputMedia = lookup_string(medias, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-portrait", 3) == 0)
    OutputOrient = PCL_PORTRAIT;
   else if (compare_string(argv[i], "-landscape", 2) == 0)
    OutputOrient = PCL_LANDSCAPE;
   else if (compare_string(argv[i], "-pages", 3) == 0)
    {
     i ++;
     if ((OutputPages = argv[i]) == NULL)
      usage();
    }
   else if (compare_string(argv[i], "-flip", 2) == 0)
    {
     i ++;
     if ((OutputFlip = lookup_string(flips, argv[i])) < 0)
      usage();
    }
   else if (compare_string(argv[i], "-gamma", 2) == 0)
    {
     i ++;
     if (argv[i] == NULL ||
         (OutputGamma = atof(argv[i])) < 0.0 ||
         OutputGamma > 20.0)
      usage();
    }
   else
    usage();
  };
}


/*
 * 'set_defaults()' - Set defaults from current options.
 */

static PRINTER *
set_defaults(RGB_IMAGE *i) /* I - Image to be printed */
{
 int     x;                /* Looping var... */
 PRINTER *printer;         /* Currently selected printer... */


 printer = NULL;
 for (x = 0; x < (sizeof(PrinterModels) / sizeof(PRINTER)); x ++)
  if (PrinterModels[x].model == PrinterModel)
   {
    printer = PrinterModels + x;
    break;
   };

 if (printer == NULL)
  {
   fputs("\atopcl: Internal program error during search for printer model!\n", stderr);
   exit(1);
  };

 if (OutputType == PCL_AUTO)
  OutputType = printer->type;

 if (OutputMode == PCL_AUTO)
  OutputMode = printer->mode;

 if (OutputDpi == PCL_AUTO)
  OutputDpi = printer->dpi;

 if (OutputOrient == PCL_AUTO)
  {
  /*
   * If no orientation has be selected, orient the image to have the long
   * side lie on the vertical edge (to fill the page as well as possible).
   */

   if (i->width > i->height)
    OutputOrient = PCL_LANDSCAPE;
   else
    OutputOrient = PCL_PORTRAIT;
  };

 switch (PaperSize)
  {
   case PCL_LETTER_SIZE :
       PaperWidth  = printer->letter_width;
       PaperHeight = printer->letter_height;
       break;
   case PCL_LEGAL_SIZE :
       PaperWidth  = printer->legal_width;
       PaperHeight = printer->legal_height;
       break;
   case PCL_EXECUTIVE_SIZE :
       PaperWidth  = printer->exec_width;
       PaperHeight = printer->exec_height;
       break;
   case PCL_A4_SIZE :
       PaperWidth  = printer->a4_width;
       PaperHeight = printer->a4_height;
       break;
  };

 return (printer);
}


/*
 * 'main()' - Get options and output as many pages as necessary...
 */

void
main(int  argc,        /* I - Number of command-line arguments */
     char *argv[])     /* I - Command-line arguments */
{
 RGB_IMAGE *i;         /* Image to convert */
 int       hpage,      /* Horizontal page count */
           vpage;      /* Vertical page count */
 int       num_hpages, /* Number of pages vertically */
           num_vpages; /* Number of pages horizontally */
 int       copy,       /* Current copy */
           num_copies; /* Number of copies to make */
 PRINTER   *printer;   /* Current printer */


/*
 * First check for command-line args and parse-'em...
 */

 if (argc < 2)
  usage();

 i = LoadImage(argv[1]);
#ifdef DEBUG
 fputs("Image loaded...\n", stderr);
#endif /* DEBUG */

 parse_options(argc - 2, argv + 2);

/*
 * Set any remaining defaults and prepare the image for output...
 */

 printer = set_defaults(i);
#ifdef DEBUG
 fputs("Defaults set...\n", stderr);
#endif /* DEBUG */

 if (OutputType == PCL_BW)
  GreyImage(i);

 AdjustImage(i, OutputGamma);
#ifdef DEBUG
 fputs("Image prepped...\n", stderr);
#endif /* DEBUG */


/*
 * Figure out how many pages per image to output...
 */

 if (sscanf(OutputPages, "%dx%d", &num_hpages, &num_vpages) < 2)
  { /* Only one axis specified; duplicate... */
   num_vpages = num_hpages;
  };

/*
 * Initialize the printer as necessary...
 */

 printf("\033E");                     /* Reset the printer */

 if (printer->copies)
  {
   printf("\033&l%dX", OutputCopies); /* Have the printer do the copies */
   num_copies = 1;                    /* Dither the image once */
  }
 else
  {
   num_copies = OutputCopies;         /* Dither the image 'num_copies' times */
  };

 printf("\033&l%dA", PaperSize);      /* Set paper size... */

 if (PaperSource != PCL_DEFAULT_TRAY)
  printf("\033&l%dH", PaperSource);   /* Get media from specified source... */

 printf("\033*t%dR", OutputDpi);      /* Set graphics resolution */

/*
 * Now output each page & copy as necessary...
 */

 for (copy = 0; copy < num_copies; copy ++)
  for (hpage = 0; hpage < num_hpages; hpage ++)
   for (vpage = 0; vpage < num_vpages; vpage ++)
    OutputImage(i, hpage * i->width / num_hpages,
                vpage * i->height / num_vpages,
                (hpage + 1) * (i->width - 1) / num_hpages,
                (vpage + 1) * (i->height - 1) / num_vpages);

 printf("\033E");                     /* Reset the printer */

 FreeImage(i);                        /* Free up any memory used before exit */
}


/*
 * End of "$Id: topcl.c,v 1.5 93/12/22 16:50:39 mike Exp Locker: mike $".
 */
